/* CHAP04.C  (Sample Program for "Menus a la Carte") */

#include "windows.h"
#include "chap04.h"

/* ******************************************************************* *
 *                                                                     *
 *                         WinMain Procedure                           *
 *                                                                     *
 * ******************************************************************* */

int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;               /* current program instance */
HANDLE hPrevInstance;           /* previous program instance */
LPSTR lpCmdLine;                /* command line string */
int nCmdShow;                   /* icon or window appearance */
{
     MSG msg;
     HWND hwnd;
     HANDLE hAccel;

     /* Call window registration routine */

     if(!hPrevInstance && !InitODwin(hInstance))
         return(FALSE);

    /* Initialize new instance of a window */

     if(!(hwnd = CreateODwindow(hInstance, nCmdShow)))
        return(FALSE);

    /* Load keyboard accelerators */

     hAccel = LoadAccelerators(hInstance, "Ch4Accel");

    /* Revised message loop, now with accelerators */

    while (GetMessage(&msg, NULL, NULL, NULL))
        if(!TranslateAccelerator(hwnd, hAccel, &msg))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    return(msg.wParam);
}

/* ******************************************************************* *
 *                                                                     *
 *                       Window Procedures                             *
 *                                                                     *
 * ******************************************************************* */

BOOL FAR PASCAL About(hAbout, msg, wParam, lParam)
HWND hAbout;
unsigned msg;
WORD wParam;
LONG lParam;
{
    switch(msg)
    {
        case WM_INITDIALOG:
            AdjustWINcursor(hAbout);
            SetFocus(GetDlgItem(hAbout, IDOK));
            return(FALSE);

        case WM_COMMAND:
            if (wParam == IDOK || wParam == IDCANCEL)
            {
                EndDialog(hAbout, TRUE);
                return(TRUE);
            }
            break;
    }
    return(FALSE);
}

long FAR PASCAL Pinup2Procedure(hwnd, msg, wParam, lParam)
HWND hwnd;
unsigned msg;
WORD wParam;
LONG lParam;
{
    static BOOL bMoving = FALSE;

    switch(msg)
    {
        case WM_COMMAND:
            if(wParam > IDM_BEGIN && wParam < IDM_END)
                ProcessWMcommand(hwnd, wParam);
            break;

        case WM_NCRBUTTONDOWN:
            bMoving = MovePMpinup(hwnd, msg);
            break;

        case WM_MOUSEMOVE:
            if(bMoving)
                MovePMpinup(hwnd, msg);
            break;

        case WM_RBUTTONUP:
            bMoving = MovePMpinup(hwnd, msg);
            SendMessage(hwnd, WM_NCPAINT, 0, 0L);
            break;
    }

    return(DefWindowProc(hwnd, msg, wParam, lParam));
}

long FAR PASCAL PopupProcedure(hwnd, msg, wParam, lParam)
HWND hwnd;
unsigned msg;
WORD wParam;
LONG lParam;
{
    static BOOL bAbout = FALSE,
                bPopup = FALSE;

    switch(msg)
    {
        case WM_COMMAND:
            if(wParam > IDM_BEGIN && wParam < IDM_END)
            {
                bAbout = (wParam == IDM_ABOUT) ? TRUE : FALSE;
                ProcessWMcommand(hwnd, wParam);
            }
            break;

    /* The following message trio lets you to establish flag
       values to control whether AdjustPMcusor is called.
       WM_INITMENUPOPUP occurs only if the user clicks on a
       top-level menu; WM_MENUSELECT occurs numerous times
       through the menu selection process; and WM_NCACTIVATE
       occurs once at the end of the process (thus it is
       place for resetting flags. */

        case WM_INITMENUPOPUP:
            bPopup = TRUE;
            break;

        case WM_MENUSELECT:
            if(!bAbout && bPopup)
                AdjustPMcursor(hwnd, wParam, lParam);
            break;

        case WM_NCACTIVATE:
            bAbout = FALSE;
            bPopup = FALSE;
            break;
    }

    return(DefWindowProc(hwnd, msg, wParam, lParam));
}

long FAR PASCAL WindowProcedure(hwnd, msg, wParam, lParam)
HWND hwnd;
unsigned msg;
WORD wParam;
LONG lParam;
{
    HBITMAP hBmOn, hBmOff;
    static BOOL bMenu = FALSE;


    switch(msg)
    {
        case WM_CREATE:

            idTotal = IDM_DYNAMIC; /* DM counter */

            LoadBMitem(hwnd, IDM_COPY, "BMcopy");
            LoadBMitem(hwnd, IDM_CUT,  "BMcut");
            LoadBMitem(hwnd, IDM_PASTE, "BMpaste");
            LoadBMitem(hwnd, IDM_ABOUTBM, "BMabout");

            hBmOn = LoadBMcheck(hwnd, "XCheck");
            SetBMcheck(hwnd, IDM_DEMOA1, 4, NULL, hBmOn);
            hBmOff = LoadBMcheck(hwnd, "ChkOff");
            hBmOn = LoadBMcheck(hwnd, "ChkOn");
            SetBMcheck(hwnd, IDM_DEMOB1, 4, hBmOff, hBmOn);
            break;

        case WM_TIMER:
            break;

        case WM_COMMAND:
            if(wParam > IDM_BEGIN && wParam < IDM_END)
                ProcessWMcommand(hwnd, wParam);
            break;

        case WM_LBUTTONDOWN:
            ShowPMfloat(hwnd, lParam);
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

    }
    return(DefWindowProc(hwnd, msg, wParam, lParam));
}
/* ******************************************************************* *
 *                                                                     *
 *                     Initialize/Create Routines                      *
 *                                                                     *
 * ******************************************************************* */

BOOL InitODwin(hInstance)
HANDLE hInstance;
{
    WNDCLASS  wc;

    wc.style = NULL;
    wc.lpfnWndProc = WindowProcedure;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "PrimerIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "Ch4Menu";
    wc.lpszClassName = "Ch4Class";

    return(RegisterClass(&wc));
}

BOOL InitPMpinup1()            /* FILENAME: INPINUP1.C */
{
    WNDCLASS  wc;

    wc.style = NULL;
    wc.lpfnWndProc = PopupProcedure;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInst;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = NULL;
    wc.lpszMenuName =  "PinupMenu1";
    wc.lpszClassName = "PopupClass1";

    return(RegisterClass(&wc));
}

BOOL InitPMpinup2()            /* FILENAME: INPINUP2.C */
{
    WNDCLASS  wc;

    wc.style = NULL;
    wc.lpfnWndProc = Pinup2Procedure;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInst;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = NULL;
    wc.lpszMenuName =  "PinupMenu2";
    wc.lpszClassName = "PopupClass2";

    return(RegisterClass(&wc));
}

HWND CreateODwindow(hInstance, nCmdShow)
HANDLE hInstance;
int nCmdShow;
{
    HWND hwnd;

    hInst = hInstance;

    hwnd = CreateWindow(
        "Ch4Class",
        "Chapter 4: Menus a la Carte",
        dwStyle,
        0,
        0,
        500,
        400,
        NULL,
        NULL,
        hInstance,
        NULL);

    if(!hwnd)
        return(FALSE);

    ShowWindow(hwnd,nCmdShow);
    UpdateWindow(hwnd);
    return(hwnd);
}

/* ******************************************************************* *
 *                                                                     *
 *                       System Menu Routines                          *
 *                                                                     *
 * ******************************************************************* */

WORD AppendSMitem(hwnd, lpNewItem)     /* FILENAME: APPSM.C */
HWND hwnd;
LPSTR lpNewItem;
{
    HMENU hMenu;
    static BOOL bFlag = TRUE;
    static WORD id = IDM_SYSMENUITEM;

    hMenu = GetSystemMenu(hwnd, FALSE);

    /* Check for first insertion, so as to add seperator bar */

    if(bFlag)
       AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);

    AppendMenu(hMenu, MF_STRING, id, lpNewItem);
    bFlag = FALSE;
    return(id++);
}

BOOL DeleteSMitem(hwnd, nPos)          /* FILENAME: DELSM.C */
HWND hwnd;
WORD nPos;
{
     int i;
     WORD wCount;
     HMENU hMenu;

     hMenu = GetSystemMenu(hwnd, FALSE);

 /*  Get the intial menu item count for separator bar checks
     that occur after the deletion. Note that the menu item
     count is not zero-based as is MF_BYPOSITION. Also note
     that MF_SEPARATOR is equal to zero for ID purposes. */

     wCount = GetMenuItemCount(hMenu);

     if(nPos >= wCount) return(FALSE);

     DeleteMenu(hMenu, nPos, MF_BYPOSITION);

     for(i=1; i < (int)wCount; i++)
         if(GetMenuItemID(hMenu, i) == 0)
             if(GetMenuItemID(hMenu, i-1) == 0)
                 DeleteMenu(hMenu, i, MF_BYPOSITION);

     if(nPos == wCount-1)
     {
         MessageBeep(0);
         if(GetMenuItemID(hMenu, nPos-1) == 0)
            DeleteMenu(hMenu, nPos-1, MF_BYPOSITION);
     }

     return(TRUE);
}

void DeleteSMseps(hwnd)                /* FILENAME: DELSMSP.C */
HWND hwnd;
{
     int i;
     WORD wCount;
     HMENU hMenu;

     hMenu = GetSystemMenu(hwnd, FALSE);
     wCount = GetMenuItemCount(hMenu);

     for(i=0; i < (int)wCount; i++)
         if(GetMenuItemID(hMenu, i) == 0)
                 DeleteMenu(hMenu, i, MF_BYPOSITION);
}

WORD InsertSMitem(hwnd, nPos, lpNewItem)   /* FILENAME: INSSM.C */
HWND hwnd;
WORD nPos;
LPSTR lpNewItem;
{
    BOOL bRet;
    HMENU hMenu;
    static int id = IDM_SYSMENUITEM;

    hMenu = GetSystemMenu(hwnd, FALSE);
    bRet = InsertMenu(hMenu, nPos,
                 MF_BYPOSITION | MF_STRING,
                 id, lpNewItem);

    return( (bRet) ? id++ : -1);

}

/* ******************************************************************* *
 *                                                                     *
 *                        Dynamic Menu Routines                        *
 *                                                                     *
 * ******************************************************************* */

WORD AppendDMitem(hDynMenu, lpNewItem)     /* FILENAME: APPDM.C */
HMENU hDynMenu;
LPSTR lpNewItem;
{
    int id;
    BOOL bRet;
    WORD wFlags = MF_ENABLED | MF_BYCOMMAND;

    id = idTotal;
    bRet = AppendMenu(hDynMenu, wFlags, id, lpNewItem);
    arrayID[idTotal] = id;         /* Update globals */
    ++idTotal;

    return( (bRet) ? id : -1);
}

BOOL DeleteDMitem(hDynMenu, id)            /* FILENAME: DELDM.C */
HMENU hDynMenu;
WORD id;
{
    int i;
    BOOL bRet, bDel=FALSE;

    bRet = DeleteMenu(hDynMenu, id, MF_BYCOMMAND);
    if(!bRet)
        return(FALSE);

    for(i=IDM_DYNAMIC; i<=IDM_DYNAMIC+idTotal; i++)
    {
        if(!bDel)
            if(arrayID[i]=id)
                bDel = TRUE;
        if(bDel)
            arrayID[i]==arrayID[i+1];
    }
    --idTotal;
    return(TRUE);
}

HMENU LoadDM(hwnd, lpNewMenu)              /* FILENAME: LOADDM.C */
HWND hwnd;
LPSTR lpNewMenu;
{
    HMENU hDynMenu, hMenu;

    if(!(hDynMenu = CreateMenu()))
        return(0);
    hMenu = GetMenu(hwnd);
    AppendMenu(hMenu, MF_POPUP | MF_ENABLED, hDynMenu, lpNewMenu);
    DrawMenuBar(hwnd);
    return(hDynMenu);
}

/* ******************************************************************* *
 *                                                                     *
 *                        Bitmap Menu Routines                         *
 *                                                                     *
 * ******************************************************************* */


HBITMAP LoadBMcheck(hwnd, lpBitmap)        /* FILENAME: LOADBMCH.C */
HWND hwnd;
LPSTR lpBitmap;
{
    LONG lSize;
    int nWidth, nHeight;
    HDC hdc, hdcOld, hdcNew;
    HBITMAP hBm, hBmX, hBmOld, hBmNew;

    hBm = LoadBitmap(hInst, lpBitmap);
    lSize = GetMenuCheckMarkDimensions();
    nWidth = (int)LOWORD(lSize);
    nHeight = (int)HIWORD(lSize);

    hdc = GetDC(hwnd);
    hdcNew = CreateCompatibleDC(hdc);
    hdcOld = CreateCompatibleDC(hdc);

    hBmX = CreateCompatibleBitmap(hdc, nWidth, nHeight);

    hBmOld = SelectObject(hdcNew, hBmX);
    hBmNew = SelectObject(hdcOld, hBm);

    StretchBlt(hdcNew, 0, 0, nWidth, nHeight,
                hdcOld, 0, 0, 64, 64, SRCCOPY);

    SelectObject(hdcNew, hBmOld);
    SelectObject(hdcOld, hBmNew);
    DeleteDC(hdcNew);
    DeleteDC(hdcOld);
    ReleaseDC(hwnd, hdc);
    return(hBmX);
}

BOOL LoadBMitem(hwnd, id, lpBitmap)        /* FILENAME: LOADBMIT.C */
HWND hwnd;
WORD id;
LPSTR lpBitmap;

{
    BOOL bRet;
    HMENU hMenu;
    HBITMAP hBitmap;

    hBitmap = LoadBitmap(hInst, lpBitmap);
    hMenu = GetMenu(hwnd);
    bRet = ModifyMenu(hMenu, id,
               MF_BYCOMMAND | MF_BITMAP,
               id, (LPSTR)MAKELONG(hBitmap,0));

    return(bRet);

}

void SetBMcheck(hwnd, id, iTotal, hBmOff, hBmOn)  /* FILENAME: SETBMCH.C */
HWND hwnd;
WORD id;
int iTotal;
HBITMAP hBmOff;
HBITMAP hBmOn;
{
    HMENU hMenu;
    int i;

    hMenu = GetMenu(hwnd);
    for(i=0; i<=iTotal; i++)
        SetMenuItemBitmaps(hMenu, id+i,
                             MF_BYCOMMAND,
                             hBmOff, hBmOn);
}

/* ******************************************************************* *
 *                                                                     *
 *                    Popup and Pinup Menu Routines                    *
 *                                                                     *
 * ******************************************************************* */

void AdjustPMcursor(hwnd, wParam, lParam)    /* FILENAME: ADJPMCUR.C */
HWND hwnd;
WORD wParam;
LONG lParam;
{
    switch(LOWORD(lParam))
    {
        case MF_CHECKED:
        case MF_DISABLED:
        case MF_GRAYED:
        case MF_SYSMENU:             /* for any modified pinup */
            return;

        default:
            break;
    }

    if(lParam == 0x0000FFFF)         /* WM_MENUSELECT completion */
        SetWINfocus(hwnd);
}

BOOL MovePMpinup(hwnd, msg)            /* FILENAME: MVPMPIN.C */
HWND hwnd;
unsigned msg;
{
    RECT rc;
    POINT pt;

    if(msg == WM_NCRBUTTONDOWN)
    {
        SetCapture(hwnd);
        return(TRUE);
    }

    if(msg == WM_MOUSEMOVE)
    {
        GetWindowRect(hwnd, &rc);
        GetCursorPos(&pt);
        MoveWindow(hwnd, pt.x, pt.y,
                     rc.right - rc.left,
                     rc.bottom - rc.top,
                     TRUE);
        return(TRUE);
    }

    if(msg == WM_RBUTTONUP)
    {
        ReleaseCapture(hwnd);
        return(FALSE);
    }

    return(FALSE);
}

BOOL ShowPMfloat(hwnd, lParam)         /* FILENAME: SHPMFLOT.C */
HWND hwnd;
LONG lParam;
{
    RECT rc;
    POINT pt;
    HMENU hMenu, hPop;

    GetClientRect(hwnd, &rc);

    if(PtInRect((LPRECT)&rc, MAKEPOINT(lParam)))
    {
        MessageBeep(0);
        hMenu = LoadMenu(hInst, "PopupMenu");
        hPop = GetSubMenu(hMenu, 0);
        pt = MAKEPOINT(lParam);
        ClientToScreen(hwnd, (LPPOINT)&pt);
        if(!(TrackPopupMenu(hPop, 0, pt.x, pt.y, 0, hwnd, NULL)))
            return(FALSE);
    }
    DestroyMenu(hMenu);
    return(TRUE);
}


HWND ShowPMpinup1(hwnd)                /* FILENAME: SHPMPIN1.C */
HWND hwnd;
{
    HWND hwndPopup;
    int cy;

    cy = GetSystemMetrics(SM_CYSIZE)
         + GetSystemMetrics(SM_CYBORDER)
         + GetSystemMetrics(SM_CYMENU);

    InitPMpinup1();

    hwndPopup = CreateWindow(
        "PopupClass1",
        "Pinup Menu",
        WS_POPUP | WS_BORDER | WS_CAPTION,
        200, 200, 150, cy,
        hwnd, NULL, hInst, NULL);

    if(!hwndPopup) return(NULL);

    ShowWindow(hwndPopup, SW_SHOWNORMAL);
    UpdateWindow(hwndPopup);
    return(hwndPopup);
}

HWND ShowPMpinup2(hwnd)                /* FILENAME: SHPMPIN2.C */
HWND hwnd;
{
    HWND hwndPopup;
    int cy;

    cy = GetSystemMetrics(SM_CYBORDER)
         + GetSystemMetrics(SM_CYMENU);

    InitPMpinup2();

    hwndPopup = CreateWindow(
        "PopupClass2",
        "Pinup Menu 2",
        WS_POPUP,
        200, 200, 66, cy,
        hwnd, NULL, hInst, NULL);

    if(!hwndPopup) return(NULL);

    ShowWindow(hwndPopup, SW_SHOWNORMAL);
    UpdateWindow(hwndPopup);
    return(hwndPopup);
}


/* ******************************************************************* *
 *                                                                     *
 *                      User Menu Routines  (UM)                       *
 *                                                                     *
 * ******************************************************************* */

HMENU ChangeUMall(hwnd, lpMenu)        /* FILENAME: CHANGEUM.C */
HWND hwnd;
LPSTR lpMenu;
{
     HMENU hMenu;
     static HMENU hPrevMenu;
     static BOOL bRestore=FALSE;

  /* This routine is called as a result of two menu commands,
     namely IDM_CHANGEUMALL to change all menus to alternative
     set; and IDM_RESTOREUMALL to restore original menus. */

     if(lpMenu == NULL && bRestore)
     {
         SetMenu(hwnd, hPrevMenu);
         bRestore = TRUE;
     }

     hPrevMenu = GetMenu(hwnd);

     if(!(hMenu = LoadMenu(hInst, lpMenu)))
         return(NULL);

     SetMenu(hwnd, hMenu);
     DrawMenuBar(hwnd);

     bRestore = TRUE;
     return(hMenu);
}

void CheckUMoff(hwnd, wItem)           /* FILENAME: CHUMOFF.C */
HWND hwnd;
WORD wItem;
{
    HMENU hMenu;

    hMenu = GetMenu(hwnd);
    CheckMenuItem(hMenu, wItem, MF_UNCHECKED);
}

void CheckUMon(hwnd, wItem)            /* FILENAME: CHUMON.C */
HWND hwnd;
WORD wItem;
{
    HMENU hMenu;

    hMenu = GetMenu(hwnd);
    CheckMenuItem(hMenu, wItem, MF_CHECKED);
}

void CheckUMonoff(hwnd, n, wItem)          /* FILENAME: CHUMONOF.C */
HWND hwnd;
int n;
WORD wItem;
{
    WORD wFlags;
    HMENU hMenu, hSubMenu;
    static BOOL bFirst = TRUE;
    static WORD item[10] = { -1,-1,-1,-1,-1,-1,-1,-1,-1,-1};

 /* This routine assumes that the intial state of each menu item
    is unchecked (you haven't used CHECKED in the .RC file).     */

    hMenu = GetMenu(hwnd);
    if(item[n] == -1)
    {
        CheckMenuItem(hMenu, wItem, MF_CHECKED);
        item[n] = wItem;
        return;
    }

    hSubMenu = GetSubMenu(hMenu, n);
    wFlags = GetMenuState(hSubMenu, item[n], MF_BYPOSITION);
    if(wFlags & MF_CHECKED)
    {
        CheckMenuItem(hMenu, wItem, MF_CHECKED);
        CheckMenuItem(hMenu, item[n], MF_UNCHECKED);
    }
    else
    {
        CheckMenuItem(hMenu, wItem, MF_UNCHECKED);
        CheckMenuItem(hMenu, item[n], MF_CHECKED);
    }

    item[n] = wItem;
}



void CheckUMtoggle(hwnd, nPos, wItem)  /* FILENAME: CHUMTOG.C */
HWND hwnd;
int nPos;
WORD wItem;
{
    int i;
    HMENU hMenu, hSubMenu;
    WORD wCount, wCurItem, wFlags,
         wChecked = MF_CHECKED | MF_BYPOSITION,
         wUnchecked  = MF_UNCHECKED | MF_BYPOSITION;

    hMenu = GetMenu(hwnd);
    hSubMenu = GetSubMenu(hMenu, nPos);
    wCount = GetMenuItemCount(hSubMenu);

    for(i=0; i < wCount; i++)
    {
       wCurItem = GetMenuItemID(hSubMenu, i);
       if(wItem != wCurItem)
       {
           wFlags = GetMenuState(hSubMenu, (WORD)i, MF_BYPOSITION);
           if(wFlags & MF_CHECKED)
               CheckMenuItem(hSubMenu, (WORD)i, wUnchecked);
           else
               CheckMenuItem(hSubMenu, (WORD)i, wChecked);
       }
    }
}

void GrayUMoff(hwnd, wItem)            /* FILENAME: GRUMOFF.C */
HWND hwnd;
WORD wItem;
{
    HMENU hMenu;

    hMenu = GetMenu(hwnd);
    EnableMenuItem(hMenu, wItem, MF_ENABLED);
}

void GrayUMon(hwnd, wItem)             /* FILENAME: GRUMON.C */
HWND hwnd;
WORD wItem;
{
    HMENU hMenu;

    hMenu = GetMenu(hwnd);
    EnableMenuItem(hMenu, wItem, MF_GRAYED);
}

void GrayUMonoff(hwnd, n, wItem)          /* FILENAME: GRUMONOF.C */
HWND hwnd;
int n;
WORD wItem;
{
    int i;
    WORD wFlags;
    HMENU hMenu, hSubMenu;
    static BOOL bFirst = TRUE;
    static WORD item[10] = { -1,-1,-1,-1,-1,-1,-1,-1,-1,-1};

 /* This routine assumes that the intial state of each menu item
    is enabled (you haven't used GRAYED in the .RC file).     */

    hMenu = GetMenu(hwnd);
    if(item[n] == -1)
    {
        EnableMenuItem(hMenu, wItem, MF_GRAYED);
        item[n] = wItem;
        return;
    }

    hMenu = GetMenu(hwnd);
    hSubMenu = GetSubMenu(hMenu, n);
    wFlags = GetMenuState(hSubMenu, item[n], MF_BYPOSITION);
    if(wFlags & MF_GRAYED)
    {
        EnableMenuItem(hMenu, wItem, MF_GRAYED);
        EnableMenuItem(hMenu, item[n], MF_ENABLED);
    }
    else
    {
        EnableMenuItem(hMenu, wItem, MF_ENABLED);
        EnableMenuItem(hMenu, item[n], MF_GRAYED);
    }

    item[n] = wItem;
}

void GrayUMtoggle(hwnd, nPos, wItem)   /* FILENAME: GRUMTOG.C */
HWND hwnd;
int nPos;
WORD wItem;
{
    int i;
    HMENU hMenu, hSubMenu;
    WORD wCount, wCurItem, wFlags,
         wEnabled = MF_ENABLED | MF_BYPOSITION,
         wGrayed  = MF_GRAYED | MF_BYPOSITION;

    hMenu = GetMenu(hwnd);
    hSubMenu = GetSubMenu(hMenu, nPos);
    wCount = GetMenuItemCount(hSubMenu);

    for(i=0; i < wCount; i++)
    {
       wCurItem = GetMenuItemID(hSubMenu, i);
       if(wItem != wCurItem)
       {
           wFlags = GetMenuState(hSubMenu, (WORD)i, MF_BYPOSITION);
           if(wFlags & MF_GRAYED)
               EnableMenuItem(hSubMenu, (WORD)i, wEnabled);
           else
               EnableMenuItem(hSubMenu, (WORD)i, wGrayed);
       }
    }
}

/* ******************************************************************* *
 *                                                                     *
 *                      General Purpose Routines                       *
 *                                                                     *
 * ******************************************************************* */

void AdjustWINcursor(hwnd)             /* FILENAME: ADJWNCUR.C */
HWND hwnd;
{
    RECT rc;

    GetWindowRect(hwnd, &rc);
    SetCursorPos( rc.left + ((rc.right - rc.left) / 2),
                  rc.top + ((rc.bottom - rc.top)) / 2 );
}

void SetWINfocus(hwnd)                 /* FILENAME: SETWNFO.C */
HWND hwnd;
{
    RECT rc;
    HWND hwndParent, hwndTemp;

    hwndTemp = hwnd;
    while(hwndTemp)
    {
        hwndParent = hwndTemp;
        hwndTemp=GetParent(hwndTemp);
    }

    GetWindowRect(hwndParent, &rc);
    SetFocus(hwndParent);
    SetCursorPos(rc.left + 20, rc.top
                                + GetSystemMetrics(SM_CYSIZE)
                                + GetSystemMetrics(SM_CYBORDER)
                                + GetSystemMetrics(SM_CYMENU));
}


/* ******************************************************************* *
 *                                                                     *
 *                      WM_COMMAND Processing                          *
 *                                                                     *
 * ******************************************************************* */

BOOL ProcessWMcommand(hwnd, command)
HWND hwnd;
WORD command;
{
    FARPROC lpProc;
    char buf[25], tmp[10];
    static i=0;
    static HWND hwndPinup1, hwndPinup2;
    static HMENU hDynMenu;
    static WORD wPrevCmd;


    if(command >= IDM_DYNAMIC && command <= idTotal)
        DisplayMBinfo(hwnd, 1, "You selected a Dynamic Menu item.");

    switch(command)
    {
        case IDM_DEMOA1:
        case IDM_DEMOA2:
        case IDM_DEMOA3:
        case IDM_DEMOA4:
        case IDM_DEMOA5:
            MessageBeep(0);
            CheckUMon(hwnd, command);
            CheckUMoff(hwnd, wPrevCmd);      /* One way of doing it  */
            wPrevCmd = command;              /* is with static value */
            break;

        case IDM_DEMOB1:
            GrayUMtoggle(hwnd, 2, command);
            break;

        case IDM_DEMOB2:
            CheckUMtoggle(hwnd, 2, command);
            break;

        case IDM_CHANGEUMALL:
            ChangeUMall(hwnd, "SecondMenu");
            break;

        case IDM_RESTOREUMALL:
            ChangeUMall(hwnd, NULL);
            break;

        case IDM_APPENDSMITEM:
            AppendSMitem(hwnd, "Option One");
            break;

        case IDM_DELETESMITEM:
            DeleteSMitem(hwnd, 8);   /* Delete Task Mgr item */
            break;

        case IDM_DELETESMSEPS:
            DeleteSMseps(hwnd);      /* Delete all separators */
            break;

        case IDM_INSERTSMITEM:
            InsertSMitem(hwnd, 4, "Center");
            break;

        case IDM_LOADDM:
            hDynMenu = LoadDM(hwnd, "NewMenu");
            break;

        case IDM_APPENDDMITEM:
            strcpy(buf, "Test #");
            strcat(buf, itoa(idTotal, tmp, 10));
            AppendDMitem(hDynMenu, buf);
            break;

        case IDM_DELETEDMITEM:
            strcpy(buf, itoa(idTotal, tmp, 10));
            DisplayMBinfo(hwnd, 1, buf);
            if(idTotal >= 0)
                DeleteDMitem(hDynMenu, IDM_DYNAMIC + i);
            i++;
            break;

        case IDM_PINUPOPEN1:
            hwndPinup1 = ShowPMpinup1(hwnd);
            GrayUMonoff(hwnd, IDP_PIN, command);
            break;

        case IDM_PINUPCLOSE1:
            DestroyWindow(hwndPinup1);
            GrayUMonoff(hwnd, IDP_PIN, command);
            break;

        case IDM_PINUPOPEN2:
            hwndPinup2 = ShowPMpinup2(hwnd);
            GrayUMon(hwnd, command);
            break;

        case IDM_PINUPCLOSE2:
            DestroyWindow(hwndPinup2);
            GrayUMoff(hwnd, IDM_PINUPOPEN2);
            break;

        case IDM_CREATEMISC:
            MessageBeep(0);
            break;

        case IDM_ABOUTBM:
        case IDM_ABOUT:
            lpProc = MakeProcInstance(About, hInst);
            DialogBox(hInst, "AboutBox", hwnd, lpProc);
            FreeProcInstance(lpProc);
            break;

        default:
            break;

        }

  /* Although redundant, the following code shows how to call
      the CheckUMonoff routine for a group of items. */

     if(command >= IDM_APPENDSMITEM
                && command <= IDM_DELETESMSEPS)
         CheckUMonoff(hwnd, IDP_SYS, command);


    return(TRUE);
}

/* ******************************************************************* *
 *                                                                     *
 *                      Routines From Other Chapters                   *
 *                                                                     *
 * ******************************************************************* */

int DisplayMBinfo(hwnd, nModal, lpText)
HWND hwnd;
int nModal;
LPSTR lpText;
{
     int nOutcome;
     WORD wType = MB_OK | MB_ICONINFORMATION;

     switch(nModal)
     {
         case 1:
             wType = wType | MB_APPLMODAL;
             break;

         case 2:
             wType = wType | MB_SYSTEMMODAL;
             break;

         default:
             wType = wType | MB_TASKMODAL;
             break;
     }

     nOutcome == MessageBox(hwnd,
                            lpText,
                            "Chapter 4: Menus a la Carte",
                            wType);

     return(nOutcome);
}

